@@ -52,11 +52,12 @@ local report_otf          = logs.reporter("fonts","otf loading")
 local fonts               = fonts
 local otf                 = fonts.handlers.otf
 
-otf.version               = 3.108 -- beware: also sync font-mis.lua and in mtx-fonts
+otf.version               = 3.112 -- beware: also sync font-mis.lua and in mtx-fonts
 otf.cache                 = containers.define("fonts", "otl", otf.version, true)
 otf.svgcache              = containers.define("fonts", "svg", otf.version, true)
 otf.pngcache              = containers.define("fonts", "png", otf.version, true)
 otf.pdfcache              = containers.define("fonts", "pdf", otf.version, true)
+otf.mpscache              = containers.define("fonts", "mps", otf.version, true)
 
 otf.svgenabled            = false
 otf.pngenabled            = false
@@ -485,22 +486,26 @@ local function copytotfm(data,cache_id)
             end
         end
         --
-        parameters.designsize    = (designsize/10)*65536
-        parameters.minsize       = (minsize   /10)*65536
-        parameters.maxsize       = (maxsize   /10)*65536
-        parameters.ascender      = abs(metadata.ascender  or 0)
-        parameters.descender     = abs(metadata.descender or 0)
-        parameters.units         = units
+        parameters.designsize = (designsize/10)*65536
+        parameters.minsize    = (minsize   /10)*65536
+        parameters.maxsize    = (maxsize   /10)*65536
+        parameters.ascender   = abs(metadata.ascender  or 0)
+        parameters.descender  = abs(metadata.descender or 0)
+        parameters.units      = units
+        parameters.vheight    = metadata.defaultvheight
         --
-        properties.space         = spacer
+        properties.space      = spacer
+        properties.format     = data.format or formats.otf
+        properties.filename   = filename
+        properties.fontname   = fontname
+        properties.fullname   = fullname
+        properties.psname     = psname
+        properties.name       = filename or fullname
+        properties.subfont    = subfont
+        --
+if not CONTEXTLMTXMODE or CONTEXTLMTXMODE == 0 then
         properties.encodingbytes = 2
-        properties.format        = data.format or formats.otf
-        properties.filename      = filename
-        properties.fontname      = fontname
-        properties.fullname      = fullname
-        properties.psname        = psname
-        properties.name          = filename or fullname
-        properties.subfont       = subfont
+end
         --
      -- properties.name          = specification.name
      -- properties.sub           = specification.sub
@@ -532,6 +537,9 @@ local converters = {
     }
 }
 
+-- We can get differences between daylight saving etc ... but it makes no sense to
+-- mess with trickery .. so be it when you use a different binary.
+
 local function checkconversion(specification)
     local filename  = specification.filename
     local converter = converters[lower(file.suffix(filename))]
