@@ -4,19 +4,13 @@
 --       AUTHOR:  Ulrike Fischer, <fischer@troubleshooting-tex.de>
 -----------------------------------------------------------------------
 
-local ProvidesLuaModule = { 
+if luaotfload_module then luaotfload_module {
     name          = "luaotfload-filelist",
-    version       = "3.15",       --TAGVERSION
-    date          = "2020-09-02", --TAGDATE
+    version       = "3.16",       --TAGVERSION
+    date          = "2020-12-31", --TAGDATE
     description   = "luaotfload submodule / filelist",
     license       = "GPL v2.0"
-}
-
-if luatexbase and luatexbase.provides_module then
-  luatexbase.provides_module (ProvidesLuaModule)
-end  
-
-
+} end
 
 --[[doc-- 
 
